#include "StdAfx.h"
#include "AppProfile.h"
#include "gnyaclock.h"
#include "vsscanf.h"

#define DEF_AND_KEY(x)	x##_DEF,x##_KEY

#define WINDOWPOS_DEF		_T("0 0 135 135")
#define WINDOWPOS_KEY		_T("window_pos")
#define COLOR_HAND_DEF		0xFF00FF
#define COLOR_HAND_KEY		_T("color_hand")
#define COLOR_DIAL_DEF		0xB92795
#define COLOR_DIAL_KEY		_T("color_dial")
#define COLOR_DOT_DEF		0xC873C8
#define COLOR_DOT_KEY		_T("color_dot")
#define CLOCK_TYPE_DEF		5
#define CLOCK_TYPE_KEY		_T("clock_type")
#define HAND_WIDTH_DEF		100
#define HAND_WIDTH_KEY		_T("hand_width")
#define BTOPMOST_DEF		0
#define BTOPMOST_KEY		_T("bTopmost")
#define BHIDEWITHMOUSE_DEF	0
#define BHIDEWITHMOUSE_KEY	_T("bHideWithMouse")
#define BTRANSPARENT_DEF	0
#define BTRANSPARENT_KEY	_T("bTransparent")
#define NTRANSPARENT_DEF	30
#define NTRANSPARENT_KEY	_T("nTransparent")
#define BPREVENTSCR_DEF		0
#define BPREVENTSCR_KEY		_T("bPreventScr")
#define NPREVENTSCR_DEF		180
#define NPREVENTSCR_KEY		_T("nPreventScr")
#define USEGDIPLUS_DEF		1
#define USEGDIPLUS_KEY		_T("useGdiPlus")
#define FRIENDLYGNYA_DEF	0
#define FRIENDLYGNYA_KEY	_T("friendlyGnya")
#define BZDIVIDE_DEF		30
#define BZDIVIDE_KEY		_T("bezierDivide")
#define COLOR_DAY_DEF		0xFFD1A6
#define COLOR_DAY_KEY		_T("color_day")
#define DRAW_DAY_DEF		1
#define DRAW_DAY_KEY		_T("draw_day")
#define GADGET_DEF			0
#define GADGET_KEY			_T("bHideGadget")
#define CIRCLE_DEF			0
#define CIRCLE_KEY			_T("bCircleWnd")
#define HEIGHTFORFONT_DEF	INT_MIN
#define HEIGHTFORFONT_KEY	_T("heightForFont")

#define OLD_BPREVENTSCR_KEY		_T("bInhScr")
#define OLD_NPREVENTSCR_KEY		_T("inhScrInterval")
#define OLD_FRIENDLYGNYA_KEY	_T("useOwnBezier")


AppProfile::AppProfile(void)
	: PrivateProfile( _T(".ini") )
{
	memset( &settings, 0, sizeof(settings) );
}

AppProfile::~AppProfile(void)
{
}

void AppProfile::ReadFormat( LPCTSTR lpAppName, LPCTSTR lpDefault, LPCTSTR lpKeyName, TCHAR *format, ... )
{
	TCHAR *value;
	FlexReadString( lpDefault, lpKeyName, &value, lpAppName );

	va_list arglist;
	va_start( arglist, format );
	_vstscanf( value, format, arglist );
	va_end( arglist );

	delete[] value;
}

void AppProfile::WriteFormat( LPCTSTR lpAppName, LPCTSTR lpDefault, LPCTSTR lpKeyName, TCHAR *format, ... )
{
	TCHAR str[256];

	va_list arglist;
	va_start( arglist, format );
	_vstprintf_s( str, _countof(str), format, arglist );
	va_end( arglist );

	WriteString( str, lpDefault, lpKeyName, lpAppName );
}

void AppProfile::ReadFont( LPCTSTR key, LOGFONT *value, LPCTSTR app )
{
	TCHAR *fontKey = NULL;
	BOOL bRet = FALSE;

	memset( value, 0, sizeof(LOGFONT) );

	FlexReadString( _T(""), key, &fontKey, app ? app : appName );
	if ( _tcslen( fontKey ) == ( sizeof(LOGFONT) + 1 ) * 2 ) {
		bRet = GetPrivateProfileStruct( app ? app : appName, key, value, sizeof(LOGFONT), profName );
	}
	else {
		int nRet = _stscanf_s( fontKey, _T("%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d"),
			&value->lfHeight,
			&value->lfWidth,
			&value->lfEscapement,
			&value->lfOrientation,
			&value->lfWeight,
			&value->lfItalic,
			&value->lfUnderline,
			&value->lfStrikeOut,
			&value->lfCharSet,
			&value->lfOutPrecision,
			&value->lfClipPrecision,
			&value->lfQuality,
			&value->lfPitchAndFamily );
		TCHAR *p = _tcsrchr( fontKey, _T(',') );
		if ( p ) {
			_tcscpy_s( value->lfFaceName, LF_FACESIZE, p + 1 );
		}
		bRet = ( ( nRet == 13 ) && ( _tcslen(value->lfFaceName) > 0 ) );
	}

	if ( !bRet ) {
		memset( value, 0, sizeof(*value) );
		value->lfHeight			= -13;	// 96dpiŖ10|Cg
		value->lfWidth			= 0;
		value->lfEscapement		= 0;
		value->lfOrientation	= value->lfEscapement;
		value->lfWeight			= FW_NORMAL;
		value->lfItalic			= FALSE;
		value->lfUnderline		= FALSE;
		value->lfStrikeOut		= FALSE;
		value->lfCharSet		= ANSI_CHARSET;
		value->lfOutPrecision	= OUT_DEFAULT_PRECIS;
		value->lfClipPrecision	= CLIP_DEFAULT_PRECIS;
		value->lfQuality		= DEFAULT_QUALITY;
		value->lfPitchAndFamily	= FF_DONTCARE|DEFAULT_PITCH;
		_tcscpy_s( value->lfFaceName, LF_FACESIZE, _T("Arial") );
	}

	delete[] fontKey;
}

void AppProfile::WriteFont( LPCTSTR key, LOGFONT *value, LPCTSTR app )
{
	LOGFONT prevLf;

	ReadFont( key, &prevLf, app );
	if ( memcmp( &prevLf, value, sizeof(LOGFONT) ) != 0 ) {
		WriteFormat( app, _T(""), key, _T("%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%s"),
			value->lfHeight,
			value->lfWidth,
			value->lfEscapement,
			value->lfOrientation,
			value->lfWeight,
			value->lfItalic,
			value->lfUnderline,
			value->lfStrikeOut,
			value->lfCharSet,
			value->lfOutPrecision,
			value->lfClipPrecision,
			value->lfQuality,
			value->lfPitchAndFamily,
			value->lfFaceName );
	}
}

BOOL AppProfile::Load(void)
{
	RenameOldKeys();

	ReadFormat( appName, DEF_AND_KEY(WINDOWPOS), _T("%d %d %d %d"), &settings.mainRect.left, &settings.mainRect.top, &settings.mainRect.right, &settings.mainRect.bottom );
	ReadFont( _T("logfont_day"), &settings.lf );

	settings.color_hand		= ReadHex( DEF_AND_KEY(COLOR_HAND) );
	settings.color_dial		= ReadHex( DEF_AND_KEY(COLOR_DIAL) );
	settings.color_dot		= ReadHex( DEF_AND_KEY(COLOR_DOT) );
	settings.clock_type 	= ReadInt( DEF_AND_KEY(CLOCK_TYPE) );
	settings.hand_width		= ReadInt( DEF_AND_KEY(HAND_WIDTH) );
	settings.bTopmost		= ReadInt( DEF_AND_KEY(BTOPMOST) );
	settings.bHideWithMouse	= ReadInt( DEF_AND_KEY(BHIDEWITHMOUSE) );
	settings.bTransparent	= ReadInt( DEF_AND_KEY(BTRANSPARENT) );
	settings.nTransparent	= ReadInt( DEF_AND_KEY(NTRANSPARENT) );
	settings.bPreventScr	= ReadInt( DEF_AND_KEY(BPREVENTSCR) );
	settings.nPreventScr	= ReadInt( DEF_AND_KEY(NPREVENTSCR) );
	settings.useGdiPlus		= ReadInt( DEF_AND_KEY(USEGDIPLUS) );
	settings.friendlyGnya	= ReadInt( DEF_AND_KEY(FRIENDLYGNYA) );
	settings.bezierDivide	= ReadInt( DEF_AND_KEY(BZDIVIDE) );
	settings.color_day		= ReadHex( DEF_AND_KEY(COLOR_DAY) );
	settings.drawDay		= ReadInt( DEF_AND_KEY(DRAW_DAY) );
	settings.heightForFont	= ReadInt( DEF_AND_KEY(HEIGHTFORFONT) );

	return TRUE;
}

BOOL AppProfile::Save(void)
{
	WriteFormat( appName, DEF_AND_KEY(WINDOWPOS), _T("%d %d %d %d"), settings.mainRect.left, settings.mainRect.top, settings.mainRect.right, settings.mainRect.bottom );
	WriteFont( _T("logfont_day"), &settings.lf, appName );

	WriteHex( settings.color_hand,		DEF_AND_KEY(COLOR_HAND) );
	WriteHex( settings.color_dial,		DEF_AND_KEY(COLOR_DIAL) );
	WriteHex( settings.color_dot,		DEF_AND_KEY(COLOR_DOT) );
	WriteInt( settings.clock_type, 		DEF_AND_KEY(CLOCK_TYPE) );
	WriteInt( settings.hand_width,		DEF_AND_KEY(HAND_WIDTH) );
	WriteInt( settings.bTopmost,		DEF_AND_KEY(BTOPMOST) );
	WriteInt( settings.bHideWithMouse,	DEF_AND_KEY(BHIDEWITHMOUSE) );
	WriteInt( settings.bTransparent,	DEF_AND_KEY(BTRANSPARENT) );
	WriteInt( settings.nTransparent,	DEF_AND_KEY(NTRANSPARENT) );
	WriteInt( settings.bPreventScr,		DEF_AND_KEY(BPREVENTSCR) );
	WriteInt( settings.nPreventScr,		DEF_AND_KEY(NPREVENTSCR) );
	WriteInt( settings.useGdiPlus,		DEF_AND_KEY(USEGDIPLUS) );
	WriteInt( settings.friendlyGnya,	DEF_AND_KEY(FRIENDLYGNYA) );
	WriteInt( settings.bezierDivide,	DEF_AND_KEY(BZDIVIDE) );
	WriteHex( settings.color_day,		DEF_AND_KEY(COLOR_DAY) );
	WriteInt( settings.drawDay,			DEF_AND_KEY(DRAW_DAY) );
	WriteInt( settings.heightForFont,	DEF_AND_KEY(HEIGHTFORFONT) );

	return TRUE;
}

void AppProfile::RenameOldKeys(void)
{
	UINT value;

	if ( ( value = ReadInt( INT_MAX, OLD_BPREVENTSCR_KEY ) ) == ReadInt( 0, OLD_BPREVENTSCR_KEY ) ) {
		WritePrivateProfileString( appName, OLD_BPREVENTSCR_KEY, NULL, profName );
		WriteInt( value, DEF_AND_KEY(BPREVENTSCR) );
	}

	if ( ( value = ReadInt( INT_MAX, OLD_NPREVENTSCR_KEY ) ) == ReadInt( 0, OLD_NPREVENTSCR_KEY ) ) {
		WritePrivateProfileString( appName, OLD_NPREVENTSCR_KEY, NULL, profName );
		WriteInt( value, DEF_AND_KEY(NPREVENTSCR) );
	}

	if ( ( value = ReadInt( INT_MAX, OLD_FRIENDLYGNYA_KEY ) ) == ReadInt( 0, OLD_FRIENDLYGNYA_KEY ) ) {
		WritePrivateProfileString( appName, OLD_FRIENDLYGNYA_KEY, NULL, profName );
		WriteInt( value, DEF_AND_KEY(FRIENDLYGNYA) );
	}
}
