#include "stdafx.h"
#include "GnyaClockFace.h"
#define _USE_MATH_DEFINES
#include <math.h>

#define	sind(ang)	sin((double) (ang)*M_PI/1800.0)
#define	cosd(ang)	cos((double) (ang)*M_PI/1800.0)

const double GnyaClockFace::cv_data[13][4] = {
    {-1500,  0.70,   1500,  0.65},		/* 12 */
    {-1350,  0.65,   1300,  0.60},		/*  1 */
    {-1200,  0.55,   1200,  0.50},		/*  2 */

    { -900,  0.45,    900,  0.40},		/*  3 */
    { -600,  0.40,    600,  0.35},		/*  4 */
    { -700,  0.30,   -450,  0.30},		/*  5 */

    {-1000,  0.25,   -800,  0.25},		/*  6 */
    { -700,  0.30,   -600,  0.30},		/*  7 */
    {  450,  0.35,   -450,  0.35},		/*  8 */

    {  900,  0.40,   -900,  0.38},		/*  9 */
    { 1350,  0.55,  -1350,  0.50},		/* 10 */
    { 2000,  0.65,  -2000,  0.60},		/* 11 */

    { 2100,  0.70,  -2100,  0.65}		/* 12 */
};


GnyaClockFace::GnyaClockFace(void)
	: prevDay( 0 )
{
}

GnyaClockFace::~GnyaClockFace(void)
{
}

void GnyaClockFace::GetBezierPoints( int tm, int width, int height, void *pxp, bool org )
{
	int i;
	int ang1, ang2;
	int d1, d2, d3;
	double hl, hs;
	double ip[4];

	width /= 2;
	height /= 2;

	if ( org ) {
		hl = 0.84;
		hs = 0.64;
	}
	else {
		hl = 0.92;
		hs = 0.92;
	}

	ang1 = tm / 12;		/* hour's angle */
	ang2 = tm % 3600;	/* minute's angle */

	d2 = ( ang1 - ang2 + 3600) % 3600;
	d1 = d2 / 300;
	d2 %= 300;
	d3 = 300 - d2;

	for( i = 0; i < 4; i++ ) {
		ip[i] = ( cv_data[d1][i] * d3 + cv_data[d1+1][i] * d2 ) / 300;
	}

	SetX( pxp, 0, (double)width  * ( 1 + sind( (double)ang1 ) * hs) );
	SetY( pxp, 0, (double)height * ( 1 - cosd( (double)ang1 ) * hs) );

	SetX( pxp, 1, (double)width  * ( 1 + sind( (double)ang1 - ip[0]) * hs * ip[1] ) );
	SetY( pxp, 1, (double)height * ( 1 - cosd( (double)ang1 - ip[0]) * hs * ip[1] ) );

	SetX( pxp, 2, (double)width  * ( 1 + sind( (double)ang2 - ip[2]) * hl * ip[3] ) );
	SetY( pxp, 2, (double)height * ( 1 - cosd( (double)ang2 - ip[2]) * hl * ip[3] ) );

	SetX( pxp, 3, (double)width  * ( 1 + sind( (double)ang2 ) * hl) );
	SetY( pxp, 3, (double)height * ( 1 - cosd( (double)ang2 ) * hl) );
}


inline static double CalcBezier( double t, double a, double b, double c, double d )
{
	double s = 1.0 - t;
	return ( s * s * s * a + 3 * s * s * t * b + 3 * s * t * t * c + t * t * t * d );
}

inline static double CalcEllipseR( double x, double cx, double dx, double y, double cy, double dy )
{
	return ( ( x - cx ) / dx ) * ( ( x - cx ) / dx ) + ( ( y - cy ) / dy ) * ( ( y - cy ) / dy );
}

void GnyaClockFace::GetBezierCurve( int div, void *pxp, void *ppp, bool friendly, int width, int height, int *startp, int *count )
{
	int start = 0;
	int end = div;
	int i;
	double t;
	double isect_t0 = 0.0, isect_t1 = 0.0;

	if ( friendly ) {
		double x, y, r, r0, t0;
		double cx = width / 2;
		double cy = height / 2;
		double dx = cx * 0.64;
		double dy = cy * 0.64;
		for ( i = 0; i < div + 1; i++ ) {
			t = (double) i / div;
			x = CalcBezier( t, GetX(pxp,0), GetX(pxp,1), GetX(pxp,2), GetX(pxp,3) );
			y = CalcBezier( t, GetY(pxp,0), GetY(pxp,1), GetY(pxp,2), GetY(pxp,3) );
			r = CalcEllipseR( x, cx, dx, y, cy, dy );
			if ( r < 1.0 ) {
				start = i - 1;
				if ( start < 0 ) start = 0;
				else {
					r0 = sqrt( r0 );
					r = sqrt( r );
					isect_t0 = t0 + ( t - t0 ) / ( r0 - r ) * ( r0 - 1.0 );
				}
				break;
			}
			else if ( r == 1.0 ) {
				start = i;
				break;
			}
			r0 = r;
			t0 = t;
		}

		dx = cx * 0.84;
		dy = cy * 0.84;
		for ( i = div; i >= 0; i-- ) {
			t = (double) i / div;
			x = CalcBezier( t, GetX(pxp,0), GetX(pxp,1), GetX(pxp,2), GetX(pxp,3) );
			y = CalcBezier( t, GetY(pxp,0), GetY(pxp,1), GetY(pxp,2), GetY(pxp,3) );
			r = CalcEllipseR( x, cx, dx, y, cy, dy );
			if ( r < 1.0 ) {
				end = i + 1;
				if ( end > div ) end = div;
				else {
					r0 = sqrt( r0 );
					r = sqrt( r );
					isect_t1 = t0 - ( t0 - t ) / ( r0 - r ) * ( r0 - 1.0 );
				}
				break;
			}
			else if ( r == 1.0 ) {
				end = i;
				break;
			}
			r0 = r;
			t0 = t;
		}
	}

	int startl	= start	+ ( isect_t0 > 0 ? 1 : 0 );
	int endl	= end	- ( isect_t1 > 0 ? 1 : 0 );
	for ( i = startl; i < endl + 1; i++ ) {
		t = (double) i / div;
		SetX( ppp, i, CalcBezier( t, GetX(pxp,0), GetX(pxp,1), GetX(pxp,2), GetX(pxp,3) ) );
		SetY( ppp, i, CalcBezier( t, GetY(pxp,0), GetY(pxp,1), GetY(pxp,2), GetY(pxp,3) ) );
	}

	if ( isect_t0 > 0 ) {
		SetX( ppp, start, CalcBezier( isect_t0, GetX(pxp,0), GetX(pxp,1), GetX(pxp,2), GetX(pxp,3) ) );
		SetY( ppp, start, CalcBezier( isect_t0, GetY(pxp,0), GetY(pxp,1), GetY(pxp,2), GetY(pxp,3) ) );
	}
	if ( isect_t1 > 0 ) {
		SetX( ppp, end, CalcBezier( isect_t1, GetX(pxp,0), GetX(pxp,1), GetX(pxp,2), GetX(pxp,3) ) );
		SetY( ppp, end, CalcBezier( isect_t1, GetY(pxp,0), GetY(pxp,1), GetY(pxp,2), GetY(pxp,3) ) );
	}

	if ( startp ) *startp = start;
	if ( count ) *count = end - start + 1;
}


bool GnyaClockFace::GetClockDot( int h, int type, int width, int height, void *prect )
{
	double dx = width * 0.02;
	double dy = height * 0.02;
	int cx = (int)( sind(h*300)*width*0.46 + width/2);
	int cy = (int)(-cosd(h*300)*height*0.46 + height/2);
	bool dr = true;

	switch( type )
	{
	case 0: dr = false;						break;	/* nothing */
	case 1: dx *= 0.7; dy *= 0.7;					/* 4 smalls */
	case 2: dr = (h%3 == 0);				break;	/* 4 larges */
	case 3: dx *= 0.7; dy *= 0.7;			break;	/* 12 smalls */
	case 4: 								break;	/* 12 larges */
	case 5: if(h%3) {dx *= 0.7; dy *= 0.7;}	break;	/* 4L + 8s */
	case 6: if(h) {dx *= 0.7; dy *= 0.7;}	break;	/* 1L + 11s */
	case 7: dr = !h;						break;	/* only the top */
	}
	if ( dr ) SetRect( prect, (double)cx - dx, (double)cy - dy, dx * 2, dy * 2 );

	return dr;
}
