#include "StdAfx.h"
#include "GnyaFaceGdip.h"
#include <windows.h>
#include <gdiplus.h>
#define _USE_MATH_DEFINES
#include <math.h>
#include "DynamicApi.h"

using namespace Gdiplus;

#define COLOR_FROM_RGB(x)	Color( GetRValue( prof.get_##x() ), GetGValue( prof.get_##x() ), GetBValue( prof.get_##x() ) )


GnyaFaceGdip::GnyaFaceGdip( AppProfile &ref )
	: prof( ref )
	, prevDay( 0 )
	, handPen( COLOR_FROM_RGB( color_hand ) )
	, backPen( COLOR_FROM_RGB( color_dial ), 2.5 )
	, dotBrush( COLOR_FROM_RGB( color_dot ) )
	, dayBrush( COLOR_FROM_RGB( color_day ) )
	, transformedPath( NULL )
	, graphics( NULL )
{
	handPen.SetStartCap( LineCapRound );
	handPen.SetEndCap( LineCapRound );
}

GnyaFaceGdip::~GnyaFaceGdip(void)
{
	if ( transformedPath ) delete transformedPath;
	if ( graphics ) delete graphics;
}

void GnyaFaceGdip::SetX( void *p, int idx, double value )
{
	PointF *outp = (PointF*)p;
	outp[idx].X = (REAL)value;
}

void GnyaFaceGdip::SetY( void *p, int idx, double value )
{
	PointF *outp = (PointF*)p;
	outp[idx].Y = (REAL)value;
}

double GnyaFaceGdip::GetX( void *p, int idx )
{
	PointF *inp = (PointF*)p;
	return inp[idx].X;
}

double GnyaFaceGdip::GetY( void *p, int idx )
{
	PointF *inp = (PointF*)p;
	return inp[idx].Y;
}

void GnyaFaceGdip::SetRect( void *p, double x, double y, double width, double height )
{
	RectF *outr		= (RectF*)p;
	outr->X			= (REAL)x;
	outr->Y			= (REAL)y;
	outr->Width		= (REAL)width;
	outr->Height	= (REAL)height;
}

void GnyaFaceGdip::UpdateSize( int width, int height, void *param )
{
	dots.clear();

	RectF rp;
	for ( int h = 0; h < 12; h++ ) {
		if ( GetClockDot( h, prof.get_clock_type(), width, height, &rp ) ) {
			dots.push_back( rp );
		}
	}

	handPen.SetWidth( (REAL)( ( ( width + height ) / 80 + 2 ) * prof.get_hand_width() / 100 ) );
	TransformDayPath( width, height );

	if ( graphics ) delete graphics;
	graphics = new Graphics( (HDC)param );
	graphics->SetSmoothingMode( SmoothingModeHighQuality );
}

void GnyaFaceGdip::UpdateSettings(void)
{
	handPen.SetColor( COLOR_FROM_RGB( color_hand ) );
	backPen.SetColor( COLOR_FROM_RGB( color_dial ) );
	dotBrush.SetColor( COLOR_FROM_RGB( color_dot ) );
	dayBrush.SetColor( COLOR_FROM_RGB( color_day ) );
	UpdateDayPath();
}

BOOL GnyaFaceGdip::UpdateDayPath( int day )
{
	BOOL bRet = FALSE;

	do {
		if ( ( day != 0 ) && ( prevDay == day ) ) break;

		if ( day != 0 ) prevDay = day;
		bRet = TRUE;

		TCHAR strDay[10];
		_stprintf_s( strDay, _countof(strDay), _T("%d"), day == 0 ? prevDay : day );

		PointF leftTop( 0.0, 0.0 );

		dayPath.Reset();

		FontFamily ff( prof.get_lf().lfFaceName );
		if ( ff.GetLastStatus() == Ok ) {

			StringFormat format;
			format.SetAlignment( StringAlignmentCenter );

			INT style = 0;
			if ( prof.get_lf().lfItalic ) style += FontStyleItalic;
			if ( prof.get_lf().lfWeight == FW_BOLD ) style += FontStyleBold;

			dayPath.AddString( strDay, -1, &ff, style, (REAL)fabs( (float)prof.get_lf().lfHeight ), leftTop, &format );
		}
		else {
			qgt.Render( prof.get_lf(), dayPath, strDay, leftTop );
		}
	} while( FALSE );

	return bRet;
}

void GnyaFaceGdip::TransformDayPath( int width, int height )
{
	RectF rectText;
	Matrix moveToCenter;

	dayPath.GetBounds( &rectText );
	moveToCenter.Translate( ( (REAL)width - rectText.Width ) / 2 - rectText.X, (REAL)height * 2 / 3 );
	moveToCenter.Scale( (REAL)height / (REAL)prof.get_heightForFont(), (REAL)height / (REAL)prof.get_heightForFont() );

	if ( transformedPath ) delete transformedPath;
	transformedPath = dayPath.Clone();
	transformedPath->Transform( &moveToCenter );
}

void GnyaFaceGdip::Draw( int width, int height, int tm, int day, void *param )
{
	PointF gxp[4];

	GetBezierPoints( tm, width, height, gxp, !prof.get_friendlyGnya() );
	if ( prof.get_friendlyGnya() ) {
		int bezierDivide = prof.get_bezierDivide();
		PointF *pp = new PointF[ bezierDivide + 1 ];

		int start, count;
		GetBezierCurve( bezierDivide, gxp, pp, true, width, height, &start, &count );
		graphics->DrawLines( &handPen, &(pp[start]), count );
		delete[] pp;
	}
	else {
		graphics->DrawBezier( &handPen, gxp[0], gxp[1], gxp[2], gxp[3] );
	}

	for ( std::vector<RectF>::iterator iter = dots.begin(); iter != dots.end(); iter++ ) {
		graphics->FillEllipse( &dotBrush, (*iter) );
	}

	if ( prof.get_drawDay() ) {

		if ( UpdateDayPath( day ) ) TransformDayPath( width, height );

		graphics->DrawPath( &backPen, transformedPath );
		graphics->FillPath( &dayBrush, transformedPath );
	}
}
