#include "StdAfx.h"
#include "resource.h"
#include "MainWindow.h"
#include "misc.h"
#include "AboutDlg.h"
#include <windowsx.h>
#include <gdiplus.h>
#include <time.h>
#include "DynamicApi.h"
#include <delayimp.h>
#include <WtsApi32.h>
#include "GnyaFaceGdi.h"
#include "GnyaFaceGdip.h"

using namespace Gdiplus;

#define TIMER_ID_DRAW	1
#define INTERVAL_DRAW	500
#define TIMER_ID_MOUSE	2
#define INTERVAL_MOUSE	100
#define INTERVAL_SHOW	1000


MainWindow::MainWindow(void)
	: m_aboutDlg( NULL )
	, memDC( NULL )
	, memBmp( NULL )
	, prevBmp( NULL )
	, m_settingDlg( NULL )
	, timerDraw( 0 )
	, timerMouse( 0 )
	, bHidden( FALSE )
	, movedOut( 0 )
	, lastMoved( 0 )
	, sessionIsLocked( FALSE )
	, gnya( NULL )
	, gnyaDeleted( TRUE )
	, backBrush( NULL )
{
	m_lpszClass = _T("MainWindow");
	m_idMenu = NULL;
	m_idIcon = IDI_TASKTRAY;
	m_hbrBackground = NULL;
	m_style = WS_POPUP | WS_DLGFRAME | WS_SIZEBOX;
	m_exStyle = WS_EX_TOOLWINDOW;
	memset( &prevPos, 0, sizeof(prevPos) );
}

MainWindow::~MainWindow(void)
{
	if ( backBrush ) DeleteObject( backBrush );
	UnregisterClass( szTitle, hInst );
}

INT_PTR CALLBACK MainWindow::WndProc( UINT message, WPARAM wParam, LPARAM lParam )
{
	switch (message)
	{
	HANDLE_MSG( m_hWnd, WM_CREATE,		Cls_OnCreate );
	HANDLE_MSG( m_hWnd, WM_COMMAND,		Cls_OnCommand );
	HANDLE_MSG( m_hWnd, WM_DESTROY,		Cls_OnDestroy );
	HANDLE_MSG( m_hWnd, WM_PAINT,		Cls_OnPaint );
	HANDLE_MSG( m_hWnd, WM_RBUTTONUP,	Cls_OnRButtonUp );
	HANDLE_MSG( m_hWnd, WM_LBUTTONDOWN,	Cls_OnLButtonDown );
	HANDLE_MSG( m_hWnd, WM_TIMER,		Cls_OnTimer );
	HANDLE_MSG( m_hWnd, WM_CLOSE,		Cls_OnClose );
	HANDLE_MSG( m_hWnd, WM_MOVE,		Cls_OnMove );
	HANDLE_MSG( m_hWnd, WM_SIZE,		Cls_OnSize );

	case WM_SIZING:				Cls_OnSizing( m_hWnd, (UINT)wParam, (LPRECT)lParam );	break;
	case WM_ENDSESSION:			PostMessage( m_hWnd, WM_CLOSE, 0, 0 );					break;
	case WM_TRAYICONMESSAGE:	Cls_OnTrayIconMessage( (UINT)lParam, (UINT)wParam );	break;

	case WM_WTSSESSION_CHANGE:
		if ( wParam == WTS_SESSION_LOCK ) sessionIsLocked = TRUE;
		else if ( wParam == WTS_SESSION_UNLOCK ) sessionIsLocked = FALSE;
		break;

	default:
		if ( message == WM_TASKBARCREATED ) Cls_OnTaskbarCreated();
		return DefWindowProc(m_hWnd, message, wParam, lParam);
	}
	return 0;
}

BOOL MainWindow::Cls_OnCreate(HWND hwnd, LPCREATESTRUCT lpCreateStruct)
{
	BOOL bRet = FALSE;
	GdiplusStartupInput gdiSI;

	do {
		if ( !notifyIcon.Init() ) break;
		if ( !notifyIcon.AddIcon( m_hWnd, szTitle, IDI_TASKTRAY ) ) break;

		__try {
			if ( SUCCEEDED( __HrLoadAllImportsForDll( "gdiplus.dll" ) ) &&
				( GdiplusStartup( &gdiToken, &gdiSI, NULL ) == Ok ) ) {
				gdiPlusEnabled = TRUE;
			}
		}
		__except( EXCEPTION_EXECUTE_HANDLER ) {
		}

		prof.Load();

		backBrush = CreateSolidBrush( prof.get_color_dial() );

		MoveWindow( hwnd,
			prof.get_mainRect().left,
			prof.get_mainRect().top,
			prof.get_mainRect().right - prof.get_mainRect().left,
			prof.get_mainRect().bottom - prof.get_mainRect().top,
			TRUE );
		SetWindowState();

		timerDraw = SetTimer( hwnd, TIMER_ID_DRAW, INTERVAL_DRAW, NULL );

		if ( GetSimpleOSVersion() > OS_2000 ) {
			bRet = wtsapi32.WTSRegisterSessionNotification( hwnd, NOTIFY_FOR_THIS_SESSION );
		}

		bRet = TRUE;

	} while( FALSE );

	return bRet;
}

void MainWindow::Cls_OnCommand(HWND hwnd, int id, HWND hwndCtl, UINT codeNotify)
{
	switch ( id )
	{
	case IDM_ABOUT:		Cmd_OnMenuAbout();		break;
	case IDM_SETTING:	Cmd_OnMenuSetting();	break;

	case IDM_TOPMOST:
		prof.set_bTopmost( !prof.get_bTopmost() );
		SetWindowState();
		break;

	case IDM_HIDE:
		prof.set_bHideWithMouse( !prof.get_bHideWithMouse() );
		break;

	case IDM_PREVENTSCR:
		prof.set_bPreventScr( !prof.get_bPreventScr() );
		break;

	case IDM_EXIT:
		PostMessage( hwnd, WM_CLOSE, 0, 0 );
		break;

	default:
		FORWARD_WM_COMMAND( hwnd, id, hwndCtl, codeNotify, DefWindowProc );
	}
}

void MainWindow::Cls_OnRButtonUp(HWND hwnd, int x, int y, UINT keyFlags)
{
	POINT pt;

	pt.x = x;
	pt.y = y;
	ClientToScreen( hwnd, &pt );
	ProcessPopup( hwnd, pt.x, pt.y );
}

void MainWindow::Cls_OnLButtonDown(HWND hwnd, BOOL fDoubleClick, int x, int y, UINT keyFlags)
{
	PostMessage( hwnd, WM_NCLBUTTONDOWN, HTCAPTION, MAKELPARAM(y,x) );
}

void MainWindow::Cls_OnClose(HWND hwnd)
{
	KillTimer( hwnd, timerDraw );
	DestroyWindow( hwnd );
}

void MainWindow::Cls_OnDestroy(HWND hwnd)
{
	RECT rect;
	GetWindowRect( hwnd, &rect );
	prof.set_mainRect( rect );
	prof.Save();

	notifyIcon.DeleteIcon( hwnd );
	if ( gdiPlusEnabled ) GdiplusShutdown( gdiToken );
	DeleteMemDC();

	if ( GetSimpleOSVersion() > OS_2000 ) {
		wtsapi32.WTSUnRegisterSessionNotification( hwnd );
	}

	gnyaDeleted = TRUE;
	if ( gnya ) delete gnya;
	gnya = NULL;

	PostQuitMessage(0);
}

BOOL MainWindow::Cls_OnTrayIconMessage( UINT mouseMsg, UINT iconId )
{
	POINT pt;
	BOOL bRet = FALSE;

	switch( mouseMsg )
	{
	case WM_RBUTTONUP:
		if ( GetSimpleOSVersion() >= OS_VISTA ) break;
	case WM_CONTEXTMENU:
		GetCursorPos( &pt );
		ProcessPopup( m_hWnd, pt.x, pt.y );
		bRet = TRUE;
		break;
	}

	return bRet;
}

void MainWindow::Cls_OnTaskbarCreated( void )
{
	notifyIcon.AddIcon( m_hWnd, szTitle, IDI_TASKTRAY );
}

void MainWindow::Cls_OnSize(HWND hwnd, UINT state, int cx, int cy)
{
	HDC hdc = GetDC( hwnd );
	CreateMemDC( hdc );
	ReleaseDC( hwnd, hdc );
}

void MainWindow::Cls_OnPaint(HWND hwnd)
{
	PAINTSTRUCT ps;
	HDC hdc;

	hdc = BeginPaint(hwnd, &ps);
	// TODO: `R[hɒǉĂ...
	CreateMemDC( hdc );
	DrawClock( hdc );
	EndPaint(hwnd, &ps);
}

void MainWindow::Cls_OnTimer(HWND hwnd, UINT id)
{
	if ( id == TIMER_ID_DRAW ) {
		HDC hdc = GetDC( hwnd );
		DrawClock( hdc );
		ReleaseDC( hwnd, hdc );

		if ( GetSimpleOSVersion() == OS_2000 ) {
			HDESK desk = user32.OpenInputDesktop(  0, FALSE, MAXIMUM_ALLOWED );
			sessionIsLocked = ( desk == NULL ) ? TRUE : FALSE;
			if ( desk ) user32.CloseDesktop( desk );
		}
	}
	else if ( id == TIMER_ID_MOUSE ) {
		POINT pt;
		GetCursorPos( &pt );
		if ( prof.get_bHideWithMouse() &&
			( pt.x >= prof.get_mainRect().left ) && ( pt.x <= prof.get_mainRect().right ) &&
			( pt.y >= prof.get_mainRect().top ) && ( pt.y <= prof.get_mainRect().bottom ) &&
			( m_aboutDlg == NULL ) && ( m_settingDlg == NULL ) ) {
				ShowWindow( m_hWnd, SW_HIDE );
				bHidden = TRUE;
		}
		else if ( bHidden ) {
			if ( movedOut == 0 ) {
				movedOut = AltGetTickCount64();
			}
			else if ( AltGetTickCount64() - movedOut > INTERVAL_SHOW ) {
				ShowWindow( m_hWnd, SW_SHOWNA );
				bHidden = FALSE;
				movedOut = 0;
			}
		}

		if ( prof.get_bPreventScr() && !sessionIsLocked ) {
			if ( ( pt.x != prevPos.x ) || ( pt.y != prevPos.y ) ) {
				lastMoved = AltGetTickCount64();
				prevPos = pt;
			}
			else if ( AltGetTickCount64() - lastMoved > ( prof.get_nPreventScr() * 1000 ) ) {
				INPUT mouseInput[] = {
					{ INPUT_MOUSE, 0, -10, 0, MOUSEEVENTF_MOVE, 0, 0 },
					{ INPUT_MOUSE, 0,  10, 0, MOUSEEVENTF_MOVE, 0, 0 },
				};
				SendInput( 2, mouseInput, sizeof(INPUT) );
				lastMoved = AltGetTickCount64();
			}
		}
	}
}

void MainWindow::Cls_OnMove(HWND hwnd, int x, int y)
{
	RECT rect;
	GetWindowRect( hwnd, &rect );
	prof.set_mainRect( rect );
}

void MainWindow::Cls_OnSizing(HWND hwnd, UINT fwSide, LPRECT pRect )
{

	if ( pRect && ( GetKeyState( VK_SHIFT ) & 0xfffe ) ) {

		int width = pRect->right - pRect->left;
		int height = pRect->bottom - pRect->top;
		int cx = pRect->left + width / 2;
		int cy = pRect->top + height / 2;

		switch( fwSide )
		{
		case WMSZ_BOTTOM:
		case WMSZ_TOP:
			pRect->left = cx - height / 2;
			pRect->right = cx + height / 2;
			break;
		case WMSZ_BOTTOMLEFT:
			if ( width > height ) {
				pRect->bottom = pRect->top + width;
			}
			else {
				pRect->left = pRect->right - height;
			}
			break;
		case WMSZ_BOTTOMRIGHT:
			if ( width > height ) {
				pRect->bottom = pRect->top + width;
			}
			else {
				pRect->right = pRect->left + height;
			}
			break;
		case WMSZ_LEFT:
		case WMSZ_RIGHT:
			pRect->top = cy - width / 2;
			pRect->bottom = cy + width / 2;
			break;
		case WMSZ_TOPLEFT:
			if ( width > height ) {
				pRect->top = pRect->bottom - width;
			}
			else {
				pRect->left = pRect->right - height;
			}
			break;
		case WMSZ_TOPRIGHT:
			if ( width > height ) {
				pRect->top = pRect->bottom - width;
			}
			else {
				pRect->right = pRect->left + height;
			}
			break;
		}

		HDC hdc = GetDC( hwnd );
		CreateMemDC( hdc );
		ReleaseDC( hwnd, hdc );

		prof.set_mainRect( *pRect );
	}
	else DefWindowProc( hwnd, WM_SIZING, (WPARAM)fwSide, (LPARAM)pRect );
}

void MainWindow::DrawClock( HDC hdc )
{
	int tm;
	struct tm current;
	time_t temptime;
	RECT rect;
	int width, height;

	if ( gnyaDeleted ) return;

	time( &temptime );
	localtime_s( &current, &temptime );
	tm = current.tm_hour * 3600 + current.tm_min * 60 + current.tm_sec;

	GetClientRect( m_hWnd, &rect );
	width = rect.right - rect.left;
	height = rect.bottom - rect.top;

	FillRect( memDC, &rect, backBrush );

	if ( gnya ) gnya->Draw( width, height, tm, current.tm_mday, (void*)memDC );

	BitBlt( hdc, 0, 0, width, height, memDC, 0, 0, SRCCOPY );
}

BOOL MainWindow::CreateMemDC( HDC hdc )
{
	RECT rect;
	int width, height;
	BOOL bRet = FALSE;

	do {
		if ( memDC ) {
			SelectObject( memDC, prevBmp );
			DeleteObject( memBmp );
			DeleteObject( memDC );
		}

		GetClientRect( m_hWnd, &rect );
		width = rect.right - rect.left;
		height = rect.bottom - rect.top;

		memDC = CreateCompatibleDC( hdc );
		if ( memDC == NULL ) break;

		memBmp = CreateCompatibleBitmap( hdc, width, height );
		if ( memBmp == NULL ) {
			DeleteObject( memDC );
			break;
		}

		prevBmp = (HBITMAP)SelectObject( memDC, memBmp );

		bRet = TRUE;

	} while( FALSE );

	if ( prof.get_heightForFont() == INT_MIN ) {
		prof.set_heightForFont( height );
	}

	if ( gnya ) gnya->UpdateSize( width, height, memDC );

	return bRet;
}

void MainWindow::DeleteMemDC(void)
{
	if ( memDC ) {
		SelectObject( memDC, prevBmp );
		DeleteObject( memBmp );
		DeleteObject( memDC );

		memDC = NULL;
		prevBmp = memBmp = NULL;
	}
}

void MainWindow::Cmd_OnMenuAbout(void)
{
	if ( m_aboutDlg ) {
		m_aboutDlg->SetFocus();
	}
	else {
		m_aboutDlg = new AboutDlg(IDD_ABOUTBOX);
		m_aboutDlg->DoModal( m_hWnd );
		delete m_aboutDlg;
		m_aboutDlg = NULL;
	}
}

void MainWindow::Cmd_OnMenuSetting(void)
{
	if ( m_settingDlg ) {
		m_settingDlg->SetFocus();
	}
	else {
		LONG prevLfHeight = prof.get_lf().lfHeight;
		m_settingDlg = new SettingDlg( IDD_SETTING, prof.GetSettings() );
		if ( m_settingDlg->DoModal( m_hWnd ) == IDOK ) {
			prof.SetSettings( m_settingDlg->GetSettings() );
			if ( prevLfHeight != prof.get_lf().lfHeight ) {
				RECT rect;
				GetClientRect( m_hWnd, &rect );
				prof.set_heightForFont( rect.bottom - rect.top );
			}
			if ( backBrush ) DeleteObject( backBrush );
			backBrush = CreateSolidBrush( prof.get_color_dial() );
			SetWindowState();
		}
		delete m_settingDlg;
		m_settingDlg = NULL;
	}
}

void MainWindow::SetWindowState(void)
{
	gnyaDeleted = TRUE;
	if ( gnya ) delete gnya;
	if ( gdiPlusEnabled && prof.get_useGdiPlus() ) {
		gnya = new GnyaFaceGdip(prof);
	}
	else {
		gnya = new GnyaFaceGdi(prof);
	}

	RECT rect;
	GetClientRect( m_hWnd, &rect );
	gnya->UpdateSettings();
	gnya->UpdateSize( rect.right - rect.left, rect.bottom - rect.top, memDC );

	gnyaDeleted = FALSE;

	SetWindowPos( m_hWnd, prof.get_bTopmost() ? HWND_TOPMOST : HWND_NOTOPMOST, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE );

	if ( GetSimpleOSVersion() >= OS_2000 ) {

		LONG_PTR wl = GetWindowLongPtr( m_hWnd, GWL_EXSTYLE );
		if ( prof.get_bTransparent() ) {
			wl |= WS_EX_LAYERED;
		}
		else {
			wl &= ~WS_EX_LAYERED;
		}
		SetWindowLongPtr( m_hWnd, GWL_EXSTYLE, wl );

		if ( prof.get_bTransparent() ) {
			BYTE alpha = (BYTE)( ( 255 * ( 100 - prof.get_nTransparent() ) ) / 100 );
			user32.SetLayeredWindowAttributes( m_hWnd, 0, alpha, LWA_ALPHA );
		}
	}

	if ( prof.get_bHideWithMouse() || prof.get_bPreventScr() ) {
		if ( timerMouse == 0 ) {
			timerMouse = SetTimer( m_hWnd, TIMER_ID_MOUSE, INTERVAL_MOUSE, NULL );
		}
	}
	else {
		if ( timerMouse ) {
			KillTimer( m_hWnd, TIMER_ID_MOUSE );
			timerMouse = 0;
		}
	}
}

void MainWindow::ProcessPopup( HWND hwnd, int x, int y )
{
	HMENU hMenu = LoadMenu( hInst, MAKEINTRESOURCE(IDC_TASKTRAY) );
	HMENU hPopup = GetSubMenu( hMenu, 0 );

	if ( prof.get_bTopmost() )			CheckMenuItem( hPopup, IDM_TOPMOST,		MF_BYCOMMAND | MF_CHECKED );
	if ( prof.get_bHideWithMouse() )	CheckMenuItem( hPopup, IDM_HIDE,		MF_BYCOMMAND | MF_CHECKED );
	if ( prof.get_bPreventScr() )		CheckMenuItem( hPopup, IDM_PREVENTSCR,	MF_BYCOMMAND | MF_CHECKED );

	SetForegroundWindow( hwnd );
	TrackPopupMenu( hPopup, TPM_LEFTALIGN | TPM_RIGHTBUTTON, x, y, 0, hwnd, NULL );
	DestroyMenu( hMenu );
}
