#include "StdAfx.h"
#include "PrivateProf.h"

PrivateProfile::PrivateProfile( TCHAR *ext )
{
	TCHAR temp[_MAX_PATH];
	TCHAR drive[_MAX_DRIVE], dir[_MAX_DIR];

	GetModuleFileName( NULL, temp, _MAX_PATH );
	_tsplitpath_s( temp, drive, _MAX_DRIVE, dir, _MAX_DIR, appName, _MAX_PATH, NULL, 0 );

	size_t lenfname = _tcslen( appName );
	if ( ( lenfname > 2 ) && ( _tcscmp( appName + lenfname - 2, _T("64") ) == 0 ) ) {
		appName[ lenfname - 2 ] = 0;
	}

#ifndef _DEBUG
	_tmakepath_s( profName, drive, dir, appName, ext );
#else
	GetCurrentDirectory( _MAX_PATH, profName );
	_tcscat_s( profName, MAX_PATH, _T("\\") );
	_tcscat_s( profName, MAX_PATH, appName );
	_tcscat_s( profName, MAX_PATH, ext );
#endif
}

PrivateProfile::~PrivateProfile(void)
{
}

DWORD PrivateProfile::FlexReadString( LPCTSTR def, LPCTSTR key, TCHAR **value, LPCTSTR app )
{
	BOOL bRet = TRUE;
	int n = 0;
	DWORD dwRet;

	*value = NULL;

	do {
		if ( *value ) {
			delete[] *value;
		}
		n += 1024;
		*value = new TCHAR[n];
		dwRet = GetPrivateProfileString( app ? app : appName, key, def, *value, n, profName );
	} while( ( dwRet + 1 ) == n );

	return dwRet;
}

BOOL PrivateProfile::WriteInt( const UINT value, const UINT def, const TCHAR *key, const TCHAR *app )
{
	BOOL bRet = TRUE;

	if ( value != ReadInt( def, key, app ? app : appName ) ) {
		TCHAR temp[256];
		_stprintf_s( temp, 256, _T("%d"), value );
		bRet = WritePrivateProfileString( app ? app : appName, key, temp, profName );
	}

	return bRet;
}

BOOL PrivateProfile::WriteHex( const UINT value, const UINT def, const TCHAR *key, const TCHAR *app )
{
	BOOL bRet = TRUE;

	if ( value != ReadHex( def, key, app ? app : appName ) ) {
		TCHAR temp[256];
		_stprintf_s( temp, 256, _T("%x"), value );
		bRet = WritePrivateProfileString( app ? app : appName, key, temp, profName );
	}

	return bRet;
}

BOOL PrivateProfile::WriteString( const TCHAR *value, const TCHAR *def, const TCHAR *key, const TCHAR *app )
{
	BOOL bRet = TRUE;
	TCHAR *temp = NULL;

	FlexReadString( def, key, &temp, app ? app : appName );

	if ( _tcscmp( value, temp ) != 0 ) {
		bRet = WritePrivateProfileString( app ? app : appName, key, value, profName );
	}
	delete[] temp;

	return bRet;
}

UINT PrivateProfile::ReadInt( const UINT def, const TCHAR *key, const TCHAR *app )
{
	return GetPrivateProfileInt( app ? app : appName, key, def, profName );
}

UINT PrivateProfile::ReadHex( const UINT def, const TCHAR *key, const TCHAR *app )
{
	TCHAR *temp = NULL;
	UINT ui;
	TCHAR strDef[256];

	_stprintf_s( strDef, 256, _T("%x"), def );

	FlexReadString( strDef, key, &temp, app ? app : appName );

	_stscanf_s( temp, _T("%x"), &ui );
	delete[] temp;

	return ui;
}

DWORD PrivateProfile::FlexReadSection( const TCHAR *section, TCHAR **value )
{
	BOOL bRet = TRUE;
	int n = 0;
	DWORD dwRet;
	TCHAR *temp = NULL;

	do {
		if ( temp ) {
			delete[] temp;
		}
		n += 1024;
		temp = new TCHAR[n];
		dwRet = GetPrivateProfileSection( section, temp, n, profName );
		dwRet++;
	} while( ( dwRet + 1 ) == n );

	if ( value ) *value = temp;

	return dwRet * sizeof(TCHAR);
}
