#include "StdAfx.h"
#include "SettingDlg.h"
#include <windowsx.h>
#include "gnyaclock.h"
#include <commctrl.h>

#define RANGE_WIDTH_LOWER		1
#define RANGE_WIDTH_UPPER		100
#define RANGE_XPARENT_LOWER		0
#define RANGE_XPARENT_UPPER		100
#define RANGE_BZDIVIDE_LOWER	1
#define RANGE_BZDIVIDE_UPPER	INT_MAX
#define RANGE_CRAMP_LOWER		1
#define RANGE_CRAMP_UPPER		INT_MAX

#define LOWER_AND_UPPER(x)		x##_LOWER, x##_UPPER
#define SET_SPIN_RANGE(x)		SendDlgItemMessage( m_hDlg, IDC_SPIN_##x, UDM_SETRANGE32, LOWER_AND_UPPER(RANGE_##x) )
#define EDIT_RANGE(x)			IDC_EDIT_##x, LOWER_AND_UPPER(RANGE_##x)


SettingDlg::SettingDlg( UINT idResource, AppProfile::SETTINGS data )
	: Dialog( idResource )
	, settings( data )
{
}

SettingDlg::~SettingDlg(void)
{
}

INT_PTR CALLBACK SettingDlg::DlgProc( UINT uMsg, WPARAM wParam, LPARAM lParam )
{
	switch( uMsg )
	{
	HANDLE_MSG( m_hDlg, WM_INITDIALOG,	Cls_OnInitDialog );
	HANDLE_MSG( m_hDlg, WM_DRAWITEM,	Cls_OnDrawItem );
	HANDLE_MSG( m_hDlg, WM_COMMAND,		Cls_OnCommand );
	}
	return (INT_PTR)FALSE;
}

void SettingDlg::Cls_OnCommand(HWND hwnd, int id, HWND hwndCtl, UINT codeNotify)
{
	switch( id )
	{
	case IDC_BTN_HAND:		SelectColor( &settings.color_hand ); break;
	case IDC_BTN_DIAL:		SelectColor( &settings.color_dial ); break;
	case IDC_BTN_DOT:		SelectColor( &settings.color_dot ); break;
	case IDC_BTN_DAYCOLOR:	SelectColor( &settings.color_day ); break;

	case IDC_BTN_DAYFONT:	OnBtnDayFont(); break;

	case IDC_CHK_XPARENT:	ToggleItem( id, IDC_EDIT_XPARENT );		break;
	case IDC_CHK_PREVENTSCR:ToggleItem( id, IDC_EDIT_CRAMP );		break;
	case IDC_CHK_DAY:		ToggleItem( id, IDC_BTN_DAYFONT ); ToggleItem( id, IDC_BTN_DAYCOLOR ); break;

	case IDC_CHK_GDIPLUS:
	case IDC_CHK_OWNBEZIER:
		EnableItem( IDC_EDIT_BZDIVIDE,
			!gdiPlusEnabled ||
			!GetCheck( IDC_CHK_GDIPLUS ) ||
			( gdiPlusEnabled && GetCheck( IDC_CHK_GDIPLUS ) && GetCheck( IDC_CHK_OWNBEZIER ) ) );
		break;

	case IDOK:
		OnBtnOk();
	case IDCANCEL:
		EndDialog( m_hDlg, id );
		break;

	default:
		if ( codeNotify == EN_CHANGE ) CheckValue( id );
		break;
	}
}

BOOL SettingDlg::Cls_OnInitDialog(HWND hwnd, HWND hwndFocus, LPARAM lParam)
{
	TCHAR strTemp[256];

	cc.lStructSize = sizeof( CHOOSECOLOR );
	cc.hwndOwner = hwnd;
	cc.lpCustColors = CustColors;
	cc.Flags = CC_RGBINIT;

	for ( int c = 0; c < 8; c++ ) {
		LoadString( hInst, c + IDS_DIAL1, strTemp, _countof(strTemp) );
		SendDlgItemMessage( hwnd, IDC_CMB_DIAL, CB_INSERTSTRING, c, (LPARAM)strTemp );
	}
	SendDlgItemMessage( hwnd, IDC_CMB_DIAL, CB_SETCURSEL, settings.clock_type, 0 );

	SetItemInt( IDC_EDIT_WIDTH,		settings.hand_width );
	SetItemInt( IDC_EDIT_XPARENT,	settings.nTransparent );
	SetItemInt( IDC_EDIT_CRAMP,		settings.nPreventScr );
	SetItemInt( IDC_EDIT_BZDIVIDE,	settings.bezierDivide );

	SetCheck( IDC_CHK_XPARENT,		settings.bTransparent );
	SetCheck( IDC_CHK_GDIPLUS,		settings.useGdiPlus );
	SetCheck( IDC_CHK_OWNBEZIER,	settings.friendlyGnya );
	SetCheck( IDC_CHK_HIDE,			settings.bHideWithMouse );
	SetCheck( IDC_CHK_TOPMOST,		settings.bTopmost );
	SetCheck( IDC_CHK_PREVENTSCR,	settings.bPreventScr );
	SetCheck( IDC_CHK_DAY,			settings.drawDay );

	EnableItem( IDC_EDIT_XPARENT,	settings.bTransparent );
	EnableItem( IDC_EDIT_BZDIVIDE,	!gdiPlusEnabled || !settings.useGdiPlus || ( gdiPlusEnabled && settings.useGdiPlus && settings.friendlyGnya ) );
	EnableItem( IDC_EDIT_CRAMP,		settings.bPreventScr );
	EnableItem( IDC_BTN_DAYFONT,	settings.drawDay );
	EnableItem( IDC_BTN_DAYCOLOR,	settings.drawDay );
	EnableItem( IDC_CHK_GDIPLUS,	gdiPlusEnabled );

	SetFontName( strTemp, _countof(strTemp), settings.lf );
	SetWindowText( GetDlgItem( hwnd, IDC_BTN_DAYFONT ), strTemp );

	if ( GetSimpleOSVersion() < OS_2000 ) {
		EnableItem( IDC_CHK_XPARENT, FALSE );
		EnableItem( IDC_EDIT_XPARENT, FALSE );
	}

	SET_SPIN_RANGE(WIDTH);
	SET_SPIN_RANGE(BZDIVIDE);
	SET_SPIN_RANGE(XPARENT);
	SET_SPIN_RANGE(CRAMP);

	return TRUE;
}

void SettingDlg::Cls_OnDrawItem(HWND hwnd, const DRAWITEMSTRUCT *lpDrawItem)
{
	HBRUSH hBrush;
	COLORREF color;

	HDC hdc = lpDrawItem->hDC;
	RECT rect = lpDrawItem->rcItem;

	switch( lpDrawItem->CtlID )
	{
	case IDC_BTN_HAND:		color = settings.color_hand;	break;
	case IDC_BTN_DIAL:		color = settings.color_dial;	break;
	case IDC_BTN_DOT:		color = settings.color_dot;		break;
	case IDC_BTN_DAYCOLOR:	color = settings.color_day;		break;

	default:
		return;
	}

	hBrush = CreateSolidBrush( color );
	FillRect( hdc, &rect, hBrush );
	DeleteObject( hBrush );

	if ( lpDrawItem->itemState & ODS_SELECTED ) {
		DrawEdge( hdc, &rect, EDGE_SUNKEN, BF_RECT );
	}
	else if ( lpDrawItem->itemState & ODS_DISABLED ) {
		DrawEdge( hdc, &rect, EDGE_ETCHED, BF_RECT );
	}
	else {
		DrawEdge( hdc, &rect, EDGE_RAISED, BF_RECT );
	}
	if ( lpDrawItem->itemState & ODS_FOCUS ) {
		RECT rectFocus = rect;
		InflateRect( &rectFocus, -4, -4 );
		DrawFocusRect( hdc, &rectFocus );
	}
}

void SettingDlg::SelectColor( COLORREF *color )
{
	cc.rgbResult = *color;

	if ( ChooseColor( &cc ) ) {
		*color = cc.rgbResult;
		InvalidateRect( m_hDlg, NULL, TRUE );
		UpdateWindow( m_hDlg );
	}
}

void SettingDlg::OnBtnDayFont(void)
{
	CHOOSEFONT cf;
	TCHAR strTemp[256];

	cf.lStructSize = sizeof(cf);
	cf.hwndOwner = m_hDlg;
	cf.lpLogFont = &settings.lf;
	cf.Flags = CF_SCREENFONTS | CF_SCALABLEONLY | CF_INITTOLOGFONTSTRUCT | CF_NOVERTFONTS;
	cf.hwndOwner = m_hDlg;
	if ( ChooseFont( &cf ) ) {
		SetFontName( strTemp, _countof(strTemp), settings.lf );
		SetItemText( IDC_BTN_DAYFONT, strTemp );
	}
}

void SettingDlg::ToggleItem( int nIDSrc, int nIDDest, BOOL rev )
{
	BOOL state = (BOOL)GetCheck( nIDSrc );
	EnableItem( nIDDest, rev ? !state : state );
}

void SettingDlg::OnBtnOk(void)
{
	settings.clock_type		= ComboBox_GetCurSel( GetDlgItem( m_hDlg, IDC_CMB_DIAL ) );
	settings.bTransparent	= (BOOL)GetCheck( IDC_CHK_XPARENT );
	settings.nTransparent	= GetItemInt( IDC_EDIT_XPARENT );
	settings.bPreventScr	= (BOOL)GetCheck( IDC_CHK_PREVENTSCR );
	settings.nPreventScr	= GetItemInt( IDC_EDIT_CRAMP );
	settings.useGdiPlus		= (BOOL)GetCheck( IDC_CHK_GDIPLUS );
	settings.friendlyGnya	= (BOOL)GetCheck( IDC_CHK_OWNBEZIER );
	settings.bezierDivide	= GetItemInt( IDC_EDIT_BZDIVIDE );
	settings.drawDay		= (BOOL)GetCheck( IDC_CHK_DAY );
	settings.bHideWithMouse	= (BOOL)GetCheck( IDC_CHK_HIDE );
	settings.bTopmost		= (BOOL)GetCheck( IDC_CHK_TOPMOST );
}

int SettingDlg::SetFontName( TCHAR *buffer, size_t sizeOfBuffer, LOGFONT lf )
{
	HDC hdc = GetDC( m_hDlg );
	int num = _stprintf_s( buffer, sizeOfBuffer, _T("%s, %d"), lf.lfFaceName, ( labs( lf.lfHeight ) * 72 + GetDeviceCaps( hdc, LOGPIXELSY ) / 2 ) / GetDeviceCaps( hdc, LOGPIXELSY ) );
	ReleaseDC( m_hDlg, hdc );

	return num;
}

void SettingDlg::CheckValue( int id )
{
	int c;
	UINT value0, value1;

	typedef struct _RANGES{
		int id;
		UINT lower;
		UINT upper;
	} RANGES;
	RANGES ranges[] = {
		{EDIT_RANGE(WIDTH)},
		{EDIT_RANGE(XPARENT)},
		{EDIT_RANGE(BZDIVIDE)},
		{EDIT_RANGE(CRAMP)},
	};

	for ( c = 0; c < _countof(ranges); c++ ) {
		if ( ranges[c].id == id ) break;
	}
	if ( c >= _countof(ranges) ) return;

	value0 = value1 = GetItemInt( id );
	if ( value1 < ranges[c].lower ) value1 = ranges[c].lower;
	if ( value1 > ranges[c].upper ) value1 = ranges[c].upper;
	if ( value0 != value1 ) SetItemInt( id, value1 );
}
